#include-once

Global $bNumberAsId = True

Global $aHotKeys[1][2] = [[0,0]]
Global $aAliases[1][2] = [[0,0]]

Func _WriteAliases($sIniFile)
    ; write aliases
    _TCHIniClearSection($sIniFile, "Alias")
    For $i=1 To $aAliases[0][0]
        _TCHIniWrite($sIniFile, "Alias", $aAliases[$i][0], $aAliases[$i][1])
    Next
EndFunc

Func _WriteHotkeys($sIniFile)
    ; write shortcuts
    _TCHIniClearSection($sIniFile, "Shortcuts")
    For $i=1 To $aHotKeys[0][0]
        _TCHIniWrite($sIniFile, "Shortcuts", $aHotKeys[$i][0], $aHotKeys[$i][1])
    Next
EndFunc

Func _CreateScripts($oXmlRoot, ByRef $aOpt)
    ; Retrieve script info and create usercommands
    $oNodes = $oXmlRoot.selectNodes('//script')
    For $oNode In $oNodes
        $sName = $oNode.getAttribute('name')
        $sFilePath = $aOpt[$IO_SCRIPT_DIR] & $sName & ".au3"
        $sTitle = IniRead($sFilePath, "info", "Title", "")
        If $sTitle = "" Then
            $oNode.setAttribute('hide', 1)
            ContinueLoop
        EndIf
        $sParam = IniRead($sFilePath, "info", "Param", "")
        $sIcon  = IniRead($sFilePath, "info", "Icon", "")
        $sSectName = $aOpt[$IO_STR_SCRIPT_PREFIX] & $sName
        $oNode.setAttribute('name', $sSectName)
        ; assign title if not overriding one exists
        If $oNode.getAttribute('title') = '' Then
            $oNode.setAttribute('title', $sTitle)
        EndIf

        $sHotKey = $oNode.getAttribute('hotkey')
        If $sHotKey <> '' Then __AddHotkey($sHotKey, $sSectName)
        $sAlias = $oNode.getAttribute('alias')
        If $sAlias <> '' Then __AddAlias($sAlias, $sSectName)

        ; now write usercommands
        IniWrite($aOpt[$IO_USERCMD_FILE], $sSectName, "cmd", $aOpt[$IO_STR_SCRIPTS_BASE] & 'Autoit3.exe "' & $aOpt[$IO_STR_SCRIPTS_BASE] & $sName & '.au3"')
        If $sTitle <> "" Then IniWrite($aOpt[$IO_USERCMD_FILE], $sSectName, "menu", $sTitle)
        If $sParam <> "" Then IniWrite($aOpt[$IO_USERCMD_FILE], $sSectName, "param", $sParam)
        If $sIcon <> "" Then IniWrite($aOpt[$IO_USERCMD_FILE], $sSectName, "button", $aOpt[$IO_STR_ICONS_BASE] & $sIcon)
    Next
EndFunc

Func _CreateUserCommands($oXmlRoot, ByRef $aOpt)
    ; get titles for existing usercommands
    $oNodes = $oXmlRoot.selectNodes('//item')
    For $oNode In $oNodes
        $sName = $oNode.getAttribute('name')
        $sTitle = $oNode.getAttribute('title')
        If $sTitle = '' Then $sTitle = $sName
        $sCmd = $oNode.getAttribute('cmd')
        $sParam = $oNode.getAttribute('param')
        $sPath = $oNode.getAttribute('path')
        $sIcon = $oNode.getAttribute('icon')
    ;    MsgBox (0, StringLen($sIcon), $sIcon)
        IniReadSection($aOpt[$IO_USERCMD_FILE], $sName)
        If @error Then 
            IniWrite($aOpt[$IO_USERCMD_FILE], $sName, "cmd", $sCmd)
            IniWrite($aOpt[$IO_USERCMD_FILE], $sName, "menu", $sTitle)
            If $sParam <> '' then IniWrite($aOpt[$IO_USERCMD_FILE], $sName, "param", $sParam)
            If $sPath <> '' then IniWrite($aOpt[$IO_USERCMD_FILE], $sName, "path", $sPath)
            If $sIcon <> '' then IniWrite($aOpt[$IO_USERCMD_FILE], $sName, "button", $sIcon)
        Else
            If $sTitle = '' Then $oNode.setAttribute('title', IniRead($aOpt[$IO_USERCMD_FILE], $sName, "menu", ""))
            If $sParam = '' Then $oNode.setAttribute('param', IniRead($aOpt[$IO_USERCMD_FILE], $sName, "param", ""))
            If $sIcon = '' Then $oNode.setAttribute('icon', IniRead($aOpt[$IO_USERCMD_FILE], $sName, "button", ""))
            If $sPath = '' Then $oNode.setAttribute('path', IniRead($aOpt[$IO_USERCMD_FILE], $sName, "path", ""))
        EndIf
    #cs
        If $oNode.getAttribute('title') = '' Then 
            $sTitle = IniRead($sUsercmdIni, $sName, "menu", "")
            If $sTitle = '' Then $sTitle = $sName
            $oNode.setAttribute('title', $sTitle)
        EndIf
    #ce
        $sHotKey = $oNode.getAttribute('hotkey')
        If $sHotKey <> '' Then __AddHotkey($sHotKey, $sName)
        $sAlias = $oNode.getAttribute('alias')
        If $sAlias <> '' Then __AddAlias($sAlias, $sName)
    Next
EndFunc

Func _CreateStdCommands($oXmlRoot, ByRef $aOpt)
    ; get standard commands
    $oNodes = $oXmlRoot.selectNodes('//cmd')
    For $oNode In $oNodes
        $sName = $oNode.getAttribute('name')
        $nCmd = Eval($sName)
        If $oNode.getAttribute('title') = '' Then
            $sTitle = IniRead($aOpt[$IO_TEMP_CMD_INI], "CommandTitles", $nCmd, "<<string not in list>>")
            $oNode.setAttribute('title', $sTitle)
        EndIf
        If $bNumberAsId Then $oNode.setAttribute('name', StringFormat("$%X", $nCmd))
        $sHotKey = $oNode.getAttribute('hotkey')
        If $sHotKey = '' Then 
            $oNode.setAttribute('hotkey', IniRead($aOpt[$IO_TEMP_CMD_INI], "DefHotkeys", $nCmd, ""))
        Else
            __AddHotkey($sHotKey, $sName)    
        EndIf
        $sAlias = $oNode.getAttribute('alias')
        If $sAlias <> '' Then __AddAlias($sAlias, $sName)
    Next
EndFunc

Func _CreateMenu($oXmlRoot, ByRef $aOpt)
    ; create menu
    $sOutput = ";Russian ()" & @CRLF & @CRLF
    $oMenuItems = $oXmlRoot.ChildNodes
    For $oMenuItem In $oMenuItems
        If $oMenuItem.nodeType <> 1 Then ContinueLoop ; only nodes
        If $oMenuItem.getAttribute('special') = 1 Then 
            $sOutput &= $oMenuItem.getAttribute('name') & @CRLF
        ElseIf $oMenuItem.getAttribute('hide') = 1 Then 
            ContinueLoop
        Else
            $sOutput &= __CreateTree($oMenuItem, 0)
        EndIf
        $sOutput &= @CRLF
    Next

    Local $hFile = FileOpen($aOpt[$IO_MNU_FILE], 2)
    FileWrite($hFile, $sOutput)
    FileClose($hFile)
EndFunc

;; Support functions

Func __CreateTree($oNode, $nLevel)
    Local $nIndent = $nLevel * 2
    Local $sIndent = ""
    While $nIndent > 0
        $sIndent &= " "
        $nIndent = $nIndent - 1
    WEnd
    Local $sTitle = $oNode.getAttribute('title')
    Local $sOutput = $sIndent & 'POPUP "' & $sTitle & '"' & @CRLF
    Local $oChilds = $oNode.ChildNodes
    For $oChild In $oChilds
        If $oChild.getAttribute('hide') = 1 Then ContinueLoop
        If $oChild.NodeName = 'group' Then 
            $sOutput &= __CreateTree($oChild, $nLevel+1)
        ElseIf $oChild.NodeName = 'separator' Then
            $sOutput &= $sIndent & '  MENUITEM SEPARATOR' & @CRLF
        Else
            $sChildTitle = $oChild.getAttribute('title')
            $sChildName = $oChild.getAttribute('name')
            $sChildHotkey = $oChild.getAttribute('hotkey')
            If $sChildHotkey <> '' Then $sChildTitle &= "\t" & $sChildHotkey ; double \\ since RegExp used
            $sOutput &= $sIndent & '  MENUITEM "' & $sChildTitle & '", ' & $sChildName & @CRLF
        EndIf
    Next
    $sOutput &= $sIndent & 'END_POPUP' & @CRLF
    Return $sOutput
EndFunc

Func __AddHotkey($sHotKey, $sCommand)
    $aHotKeys[0][0] += 1
    ReDim $aHotKeys[$aHotKeys[0][0]+1][2]
    $sHotKey = StringReplace($sHotKey, "Alt", "A")
    $sHotKey = StringReplace($sHotKey, "Ctrl", "C")
    $sHotKey = StringReplace($sHotKey, "Shift", "S")
    ; remove + between modifiers
    While 1
        $sTemp = StringReplace($sHotKey, "+", "", 1)
        If Not StringInStr($sTemp, "+") Then ExitLoop
        $sHotKey = $sTemp
    Wend
    $aHotKeys[$aHotKeys[0][0]][0] = $sHotKey
    $aHotKeys[$aHotKeys[0][0]][1] = $sCommand
EndFunc

Func __AddAlias($sAlias, $sCommand)
    $aAliases[0][0] += 1
    ReDim $aAliases[$aAliases[0][0]+1][2]
    $aAliases[$aAliases[0][0]][0] = $sAlias
    $aAliases[$aAliases[0][0]][1] = $sCommand
EndFunc