#include-once
#include "TCCommands.au3"
#include "TCHelpersLang.au3"

; ****** CONSTANTS *******
Global Const $TC_TEXT_EDITOR = @ScriptDir & "\..\Addons\Akelpad\akelpad.exe"
Global Const $TC_TEXT_EDITOR_CLASS = "[CLASS:AkelPad3]"
Global Const $TC_REGKEY_SYSTEM = "HKEY_LOCAL_MACHINE\Software\Ghisler\Total Commander"
Global Const $TC_REGKEY_USER = "HKEY_CURRENT_USER\Software\Ghisler\Total Commander"
Global Const $TC_SENDMESSAGE = 0x400 + 51 ; $WM_USER
Global Const $TC_INI = EnvGet("COMMANDER_INI")
Global Const $TC_PATH =  EnvGet("COMMANDER_PATH")
Global Const $TC_QUICK_SCRIPT_DIR =  $TC_PATH & "\Scripts\Quick\"
Global Const $TC_EXE_FULL_PATH = _TCHGetPath() & "\Totalcmd.exe" ;;;; TC_FULL_PATH
Global Const $TC_SCRIPT_COMMON_INI = @ScriptDir & "\common.ini"
Global Const $TC_LANGUAGE = _TCHLangGetID()
Global Const $TC_LANG_VAL = Eval("LANG_" & $TC_LANGUAGE)

; ****** ENUMERATIONS *******
Global Enum $SI_TITLE, $SI_PARAM, $SI_PATH, $SI_ICON, $SI_SHOW, $SI_DESCR, $SI_AUTHOR, $SI_VER, $SI_DATE

; ****** VARIABLES *******
Global $aIntenalLanguage[1][2] = [[0,0]] ; global language array for apps that need it

Func _TCHViewLogWait($sText)
    Local $sFile, $sName
    Do 
        $sName = "~"
        For $i = 0 To 6
            $sName &= Chr(Random(0x41, 0x5A))
        Next
        $sFile = @TempDir & "\" & $sName & ".log"
    Until Not FileExists($sFile)
    FileWrite($sFile, $sText)
    RunWait($TC_TEXT_EDITOR & " " & $sFile)
    FileDelete($sFile)
EndFunc

Func _TCHGetScriptProcHandle()
    Local $sTitle = "ScriptWinRandomTitle"
    For $i = 0 To 31
        $sTitle &= Chr(Random(0x41, 0x5A))
    Next
    AutoItWinSetTitle($sTitle)
    Return WinGetHandle($sTitle)
EndFunc

; retreive info from standard info block
Func _TCH_GetScriptInfo($sScriptFile, $bExtended = false)
    Local $aResult[5]
    Local $sScriptName = StringTrimRight(StringRegExpReplace($sScriptFile, "^.+\\([^\\]+)$", "\1"), 4)
    $aResult[$SI_TITLE] = IniRead($sScriptFile, "Info", "Title", $sScriptName)
    $aResult[$SI_PARAM] = IniRead($sScriptFile, "Info", "Param", "")
    $aResult[$SI_PATH]  = IniRead($sScriptFile, "Info", "Path", "")
    $aResult[$SI_ICON]  = IniRead($sScriptFile, "Info", "Icon", @AutoitExe)
    $aResult[$SI_SHOW]  = IniRead($sScriptFile, "Info", "Show", "")
    If $bExtended Then
        ReDim $aResult[9]
        $aResult[$SI_DESCR]  = IniRead($sScriptFile, "Info", "Descr", "")
        $aResult[$SI_AUTHOR] = IniRead($sScriptFile, "Info", "Author", "")
        $aResult[$SI_VER]    = IniRead($sScriptFile, "Info", "Version", "")
        $aResult[$SI_DATE]   = IniRead($sScriptFile, "Info", "Date", "")
    EndIf
    Return $aResult
EndFunc

; create button for given script
Func _TCHCreateScriptButton($sScriptFile)
    Local $aInfo = _TCH_GetScriptInfo($sScriptFile)
    Local $sCmd  = StringReplace(@AutoitExe, $TC_PATH, "%COMMANDER_PATH%") & ' "' & StringReplace($sScriptFile, $TC_PATH, "%COMMANDER_PATH%") & '"'
    Return _TCHCreateButton($sCmd, $aInfo[$SI_TITLE], $aInfo[$SI_PARAM], $aInfo[$SI_PATH], $aInfo[$SI_ICON], $aInfo[$SI_SHOW])
EndFunc

; create aux button
Func _TCHCreateButton($sCommand, $sTitle = "", $sParam = "", $sPath = "", $sIcon = @AutoitExe, $sRunType = "")
    Local $sBtn
    $sBtn = "TOTALCMD#BAR#DATA" & @CRLF ; sig
    $sBtn &= $sCommand & @CRLF ; command
    $sBtn &= $sParam & @CRLF ; params
    $sBtn &= $sIcon & @CRLF ; icon
    $sBtn &= $sTitle & @CRLF ; comment
    $sBtn &= $sPath & @CRLF ; working path
    $sBtn &= $sRunType & @CRLF ; run type
    $sBtn &= "-1" & @CRLF ; ?
    $sBtn &= @CRLF ; ?
    Return $sBtn
EndFunc

; copy text with respect of current language layout
Func _TCHCopyClipboard($sText, $sLayoutID = "00000419")
    ; comment copied wrong instead
    Local $WM_INPUTLANGCHANGEREQUEST = 0x50
    Local $hWnd = _TCHGetScriptProcHandle()
    Local $ret = DllCall("user32.dll", "long", "LoadKeyboardLayout", "str", $sLayoutID, "int", 0)
    DllCall("user32.dll", "int", "SendMessage", "hwnd", $hWnd, _
                                                "int", $WM_INPUTLANGCHANGEREQUEST, _
                                                "int", 1, _
                                                "int", $ret[0])
    ClipPut($sText)
EndFunc

Func _TCHRestart($hWnd = 0)
    If $hWnd = 0 Then $hWnd = _TCHGetActiveInstance()
    If $hWnd = 0 Then 
        MsgBox(48, $aIntenalCommonLanguage[$TC_LANG_VAL][$LANG_STR_ERROR], $aIntenalCommonLanguage[$TC_LANG_VAL][$LANG_STR_NOINSTANCE])
;        MsgBox(48, "!", "Total Commander     .")
        Exit
    EndIf
    WinClose($hWnd)
    WinWaitClose($hWnd)
    Run($TC_PATH & "\totalcmd.exe")
    If @error Then MsgBox (48, $aIntenalCommonLanguage[$TC_LANG_VAL][$LANG_STR_ERROR], $aIntenalCommonLanguage[$TC_LANG_VAL][$LANG_STR_RUNERROR])
    ;MsgBox (48, "!", "   Total Commander!")
EndFunc

Func _TCHCheckCmdLine($nReq)
    If $CmdLine[0] = 0 Then 
        MsgBox(48, $aIntenalCommonLanguage[$TC_LANG_VAL][$LANG_STR_ERROR], StringFormat($aIntenalCommonLanguage[$TC_LANG_VAL][$LANG_STR_NOPARAM], $nReq))
;        MsgBox(48, "!", StringFormat("  ,   %d .", $nReq))
        Exit
    ElseIf $CmdLine[0] < $nReq Then
        MsgBox(48, $aIntenalCommonLanguage[$TC_LANG_VAL][$LANG_STR_ERROR], StringFormat($aIntenalCommonLanguage[$TC_LANG_VAL][$LANG_STR_FEWPARAM], $CmdLine[0], $nReq))
;        MsgBox(48, "!", StringFormat(" %d  ,   %d.", $CmdLine[0], $nReq))
        Exit
    EndIf
EndFunc

Func _TCHExtractFileExt($sFile)
    Local $nSPos = StringInStr($sFile, ".", 0, -1)
    If StringInStr($sFile, "\", 0, 1, $nSPos+1) Then 
        Return ""
    Else
        Return StringMid($sFile, $nSPos+1)
    EndIf
EndFunc

Func _TCHExtractFileName($sFile, $bNoExtension = False)
    Local $nSPos = StringInStr($sFile, "\", 0, -1)
    Local $sResult, $sExt
    If $nSPos = StringLen($sFile) Then 
        Return ""
    Else
        $sResult = StringMid($sFile, $nSPos+1)
        If $bNoExtension Then
            $sExt = _TCHExtractFileExt($sResult)
            If $sExt = "" Then
                Return $sResult
            Else
                Return StringLeft($sResult, StringLen($sResult) - StringLen($sExt) - 1)
            EndIf
        Else
            Return $sResult
        EndIf    
    EndIf
EndFunc

Func _TCHExtractFilePath($sFile)
    Local $nSPos = StringInStr($sFile, "\", 0, -1)
    If $nSPos = StringLen($sFile) Then 
        Return $sFile
    Else
        Return StringLeft($sFile, $nSPos)
    EndIf
EndFunc

Func _TCHExtractBasePath($sFile)
    Local $nSPos = StringInStr($sFile, "\", 0, -1)
    If $nSPos = StringLen($sFile) Then $nSPos = StringInStr($sFile, "\", 0, -2)
    Return StringLeft($sFile, $nSPos)
EndFunc

Func _TCHGetTranslatedString($nID)
    Local $sTCPath = EnvGet("COMMANDER_PATH")
    Local $sTCConf = EnvGet("COMMANDER_INI")
    Local $sLangID = IniRead($sTCConf, "Configuration", "LanguageIni", "WCMD_RUS.LNG")
    Local $aLang = _TCHLoadFileList($sFileName)
    For $i = 1 To $aLang[0]
        $aRes = StringRegExp($aLang[0], "^" & $nID & "=(.+)", 1)
        If IsArray($aRes) Then Return $aRes[0]
    Next
    Return ""
EndFunc

; Get active instance of Total Commander (in order multiple copies runned)
Func _TCHGetActiveInstance()
    Local $aList = WinList("[CLASS:TTOTAL_CMD]")
    For $i = 1 To $aList[0][0]
        If WinActive($aList[$i][1]) Then Return $aList[$i][1]
    Next
    Return 0
EndFunc

; Post a message to given Total Commander instance
Func _TCHPostMessage($nTCMsg, $lParam = 0, $hWnd = 0)
    If $hWnd = 0 Then $hWnd = _TCHGetActiveInstance()
	Local $aResult = DllCall("user32.dll", "int", "PostMessage", "hwnd", $hWnd, "int", $TC_SENDMESSAGE, "wparam", $nTCMsg, "lparam", $lParam)
	If @error Then Return SetError(@error, @extended, "")
	Return $aResult[0]
EndFunc

; Send a message to given Total Commander instance
Func _TCHSendMessage($nTCMsg, $lParam = 0, $hWnd = 0)
    If $hWnd = 0 Then $hWnd = _TCHGetActiveInstance()
    Local $aResult = DllCall("user32.dll", "int", "SendMessage", "hwnd", $hWnd, "int", $TC_SENDMESSAGE, "wparam", $nTCMsg, "lparam", $lParam)
	If @error Then Return SetError(@error, @extended, "")
	Return $aResult[0]
EndFunc


; Try to get install path of TC (in priority: EnvVar, HKLM key, HKCU key)
Func _TCHGetPath()
    Local $sTCPath = EnvGet("COMMANDER_PATH")
    If $sTCPath = "" Then $sTCPath = RegRead($TC_REGKEY_SYSTEM, "InstallDir")
    If $sTCPath = "" Then $sTCPath = RegRead($TC_REGKEY_USER, "InstallDir")
    If $sTCPath = "" Then 
        MsgBox(48, "!", $aIntenalCommonLanguage[$TC_LANG_VAL][$LANG_STR_NOGETPATH])
;        MsgBox(48, "!", "     Total Commander.     Total Commander.")
        Exit
;        SetError(1)
    EndIf
    Return $sTCPath
EndFunc

; Load to array list of files, received with %L or %l TC macro
Func _TCHLoadFileList($sFileName)
    Local $asList[1] = [0]
    Local $hfList = FileOpen($sFileName, 0)
    While 1
        $sLine = FileReadLine($hfList)
        If @error Then Exitloop
        ReDim $asList[UBound($asList)+1]
        $asList[UBound($asList)-1] = $sLine
    Wend
    FileClose($hfList)
    $asList[0] = UBound($asList)-1
    Return $asList
EndFunc

Func _TCHAddEnv($sText)
    Return StringReplace($sText, EnvGet("COMMANDER_PATH"), "%COMMANDER_PATH%")
EndFunc

Func _TCHExpandEnv($sText)
    $aResult = StringRegExp($sText, "%(\w+)%", 3)
    If IsArray($aResult) Then
        For $i = 0 To UBound($aResult)-1
            $sText = StringReplace($sText, "%" & $aResult[$i] & "%", EnvGet($aResult[$i]))
        Next
    EndIf
    Return $sText
EndFunc

Func _FileWriteReplace($sFileName, $sText, $bBackup = False)
    If $bBackup Then FileCopy($sFileName, _TCHExtractFileName($sFileName) & ".bak", 1)
    Local $hFile = FileOpen($sFileName, 2)
    FileWrite($hFile, $sText)
    FileClose($hFile)
EndFunc


Func _IsPressed($nKey)
  Local $a
  $aR = DllCall("user32", "int", "GetAsyncKeyState", "int", "0x" & Hex($nKey, 2))
  If BitAND($aR[0], 0x8000) Then Return True
  Return False
EndFunc

; Helper function, send text to AkelPad (or other editor defined by TC_TEXT_EDITOR constant) [unfinished]
Func _SendLogToEditor($sText)
    Local $old_ref = Opt("RunErrorsFatal", 0)
    Local $old_wmt = Opt("WinTitleMatchMode", 4)
    Run($TC_TEXT_EDITOR)
    WinWait($TC_TEXT_EDITOR_CLASS, "", 3)
    ControlSetText($TC_TEXT_EDITOR_CLASS, "", 10001, $sText)
    Opt("RunErrorsFatal", $old_ref)
    Opt("WinTitleMatchMode", $old_wmt)
EndFunc

Func _FileSearch($szMask,$nOption)
    $szRoot = ""
    $hFile = 0
    $szBuffer = ""
    $szReturn = ""
    $szPathList = "*"
    Dim $aNULL[1]

    If Not StringInStr($szMask,"\") Then
         $szRoot = @SCRIPTDIR & "\"
    Else
         While StringInStr($szMask,"\")
              $szRoot = $szRoot & StringLeft($szMask,StringInStr($szMask,"\"))
              $szMask = StringTrimLeft($szMask,StringInStr($szMask,"\"))
         Wend
    EndIf
    If $nOption = 0 Then
         _FileSearchUtil($szRoot, $szMask, $szReturn)
    Else
         While 1
              $hFile = FileFindFirstFile($szRoot & "*.*")
              If $hFile >= 0 Then
                   $szBuffer = FileFindNextFile($hFile)
                   While Not @ERROR
                        If $szBuffer <> "." And $szBuffer <> ".." And _
                             StringInStr(FileGetAttrib($szRoot & $szBuffer),"D") Then _
                             $szPathList = $szPathList & $szRoot & $szBuffer & "*"
                        $szBuffer = FileFindNextFile($hFile)
                   Wend
                   FileClose($hFile)
              EndIf
              _FileSearchUtil($szRoot, $szMask, $szReturn)
              If $szPathList == "*" Then ExitLoop
              $szPathList = StringTrimLeft($szPathList,1)
              $szRoot = StringLeft($szPathList,StringInStr($szPathList,"*")-1) & "\"
              $szPathList = StringTrimLeft($szPathList,StringInStr($szPathList,"*")-1)
         Wend
    EndIf
    If $szReturn = "" Then
         $aNULL[0] = 0
         Return $aNULL
    Else
         Return StringSplit(StringTrimRight($szReturn,1),"*")
    EndIf
EndFunc

Func _FileSearchUtil(ByRef $ROOT, ByRef $MASK, ByRef $RETURN)
    $hFile = FileFindFirstFile($ROOT & $MASK)
    If $hFile >= 0 Then
         $szBuffer = FileFindNextFile($hFile)
         While Not @ERROR
              If $szBuffer <> "." And $szBuffer <> ".." Then _
                   $RETURN = $RETURN & $ROOT & $szBuffer & "*"
              $szBuffer = FileFindNextFile($hFile)
         Wend
         FileClose($hFile)
    EndIf
EndFunc

; ==================================================================================================
; Continious lists
; ==================================================================================================
Func _TCHConListRead($sIniName, $sSection, $sKeysFormat, $nStartCount = 0)
    Local $nNewIdx, $sErrStr = "<<none>>"
    Local $aKeyNames = StringSplit($sKeysFormat, "|")
    Local $nKeys = $aKeyNames[0]
    Local $aResult[1][$nKeys] = [[0]]
    Local $sRedirect = IniRead($sIniName, $sSection, "RedirectSection", $sErrStr)
    If $sRedirect <> $sErrStr Then
        $sRedirName = StringRegExpReplace($sIniName, "(.+\\)([^\\]+)", "\1")
        $sIniName = $sRedirName & $sRedirect
    EndIf
    While 1
        $sCurVal = IniRead($sIniName, $sSection, StringReplace($aKeyNames[1], "@", $aResult[0][0]+$nStartCount), $sErrStr)
        If $sCurVal <> $sErrStr Then
            $aResult[0][0] += 1
            $nNewIdx = $aResult[0][0]
            ReDim $aResult[$nNewIdx+1][$nKeys]
            $aResult[$nNewIdx][0] = $sCurVal
            For $i = 2 To $aKeyNames[0]
                $sCurVal = IniRead($sIniName, $sSection, StringReplace($aKeyNames[$i], "@", $aResult[0][0]-1+$nStartCount), $sErrStr)
                If $sCurVal <> $sErrStr Then $aResult[$nNewIdx][$i-1] = $sCurVal
            Next
        Else
            Return $aResult
        EndIf
    Wend
EndFunc

; ==================================================================================================
; Redirection-aware ini functions
; ==================================================================================================

Func _TCHIniGetRedirectName($sIniName, $sSection)
    Local $sErrStr = "<<none>>"
    Local $sBasePath = ""
    Local $sRedirect = IniRead($sIniName, $sSection, "RedirectSection", $sErrStr)
    If $sRedirect <> $sErrStr Then
        If StringInStr($sRedirect, "%") Then $sRedirect = _TCHExpandEnv($sRedirect)
        ; if path is local or UNC then return redirected path itself
        If StringInStr($sRedirect, ":") Or (StringLeft($sRedirect, 2) = "\\")  Then Return $sRedirect
        ; else assume redirect path is relative and extract source ini base path 
        $sBasePath = StringRegExpReplace($sRedirect, "(.+\\)([^\\]+)", "\1")
        $sIniName = $sBasePath & $sRedirect
    EndIf
    Return $sIniName
EndFunc

Func _TCHIniDelete($sIniName, $sSection, $sKey = "")
    $sIniName = _TCHIniGetRedirectName($sIniName, $sSection)
    If $sKey = "" Then
        Return IniDelete($sIniName, $sSection)
    Else
        Return IniDelete($sIniName, $sSection, $sKey)
    EndIf
EndFunc

Func _TCHIniClearSection($sIniName, $sSection)
    $sIniName = _TCHIniGetRedirectName($sIniName, $sSection)
    $aKeys = IniReadSection($sIniName, $sSection)
    If UBound($aKeys) = 0 Then Return
    For $i = 1 To $aKeys[0][0]
        IniDelete($sIniName, $sSection, $aKeys[$i][0])
    Next
EndFunc

Func _TCHIniRead($sIniName, $sSection, $sKey, $sDefault = '')
    $sIniName = _TCHIniGetRedirectName($sIniName, $sSection)
    Return IniRead($sIniName, $sSection, $sKey, $sDefault)
EndFunc

Func _TCHIniReadSection($sIniName, $sSection)
    $sIniName = _TCHIniGetRedirectName($sIniName, $sSection)
    Return IniReadSection($sIniName, $sSection)
EndFunc

Func _TCHIniWrite($sIniName, $sSection, $sKey, $sValue)
    $sIniName = _TCHIniGetRedirectName($sIniName, $sSection)
    Return IniWrite($sIniName, $sSection, $sKey, $sValue)
EndFunc

; ==================================================================================================
; Translations code
; ==================================================================================================
Func _TCHLangAdd($sId, $sMsg)
    $aIntenalLanguage[0][0] += 1
    Local $nIdx = $aIntenalLanguage[0][0]
    ReDim $aIntenalLanguage[$nIdx+1][2]
    $aIntenalLanguage[$nIdx][0] = $sId
    $aIntenalLanguage[$nIdx][1] = $sMsg
EndFunc

Func _TCHLangGet($sId)
    For $i = 1 To $aIntenalLanguage[0][0]
        If $aIntenalLanguage[$i][0] == $sId Then Return $aIntenalLanguage[$i][1]
    Next
    Return $aIntenalCommonLanguage[$TC_LANG_VAL][$LANG_STR_NOLANGID]
    ;"Lnguage ID is not implemented."
EndFunc

Func _TCHLangGetID()
    Local $sLng = EnvGet("COMMANDER_LANG")
    If $sLng = "" Then 
        $sLng = IniRead($TC_INI, "Configuration", "LanguageIni", "WCMD_ENG.LNG")
        $sLng = StringRegExpReplace($sLng, "(?i).+[_-]([a-zA-Z]+)\.lng", "\1")
    EndIf
    Return $sLng
EndFunc

Func _TCHLangInit($sLangID = "")
    Local $sLangFile = StringLeft(@ScriptFullPath, StringInStr(@ScriptFullPath, ".", 0, -1)-1) & ".lng"
    If $sLangID = "" Then $sLangID = $TC_LANGUAGE ; if no overriding then use 
#cs
    If $sLangID = "" Then ; auto
        $sLng = IniRead($TC_INI, "Configuration", "LanguageIni", "ENG")
        $sLng = StringRegExpReplace($sLng, "(?i).+[_-]([a-zA-Z]+)\.lng", "\1")

        If StringInStr($sLng, ".") Then
            $sLng = StringLeft($sLng, StringInStr($sLng, ".")-1)
            While StringInStr($sLng, '_')
                $sLng = StringTrimLeft($sLng, StringInStr($sLng, '_'))
            Wend
            $sLng = StringLeft($sLng, 3)
        EndIf

    EndIf
#ce
    For $i = 1 To $aIntenalLanguage[0][0]
        $aIntenalLanguage[$i][1] = IniRead($sLangFile, $sLangID, $aIntenalLanguage[$i][0], $aIntenalLanguage[$i][1])
    Next
EndFunc
